/* -*- c++ -*- */
/*
 * Copyright 2004,2005 Free Software Foundation, Inc.
 * 
 * This file is part of GNU Radio
 * 
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gr_stream_to_vector.h>
#include <gr_io_signature.h>

gr_stream_to_vector_sptr
gr_make_stream_to_vector (size_t item_size, size_t nitems_per_block)
{
  return gr_stream_to_vector_sptr (new gr_stream_to_vector (item_size, nitems_per_block));
}

gr_stream_to_vector::gr_stream_to_vector (size_t item_size, size_t nitems_per_block)
  : gr_sync_decimator ("stream_to_vector",
		       gr_make_io_signature (1, 1, item_size),
		       gr_make_io_signature (1, 1, item_size * nitems_per_block),
		       nitems_per_block)
{
}

int
gr_stream_to_vector::work (int noutput_items,
			     gr_vector_const_void_star &input_items,
			     gr_vector_void_star &output_items)
{
  size_t block_size = output_signature()->sizeof_stream_item (0);

  const char *in = (const char *) input_items[0];
  char *out = (char *) output_items[0];

  memcpy (out, in, noutput_items * block_size);

  return noutput_items;
}
